#!/bin/bash
#
# Developed by Paramvir Likhari - 2012
#
# conkywx: Program to display weather information using conky
#
# USAGE: 
# conkywx [-c path/filename] [-t path/filename] or [-d element]
# conkywx without options will launch Conkywx Weather Program Assistant
# 
# OPTIONS:
# Single options -h, -v are used on the command line
# -h   --help          Conkywx Weather Program Assistant
# -v   --version       Conkywx Weather Program Assistant
#
# Option -c and [-t, -d] can be combined on the same command line
# -c   --config        Config file to use default is /etc/conkywx/conkywx.conf
# -t   --template      Template to use
# -d   --data          For individual elements if template is not used
#
# Refer to "Conkywx Weather Program Assistant" for more details and element codes
#
###
# 

# to debug remove the comment
# set -x

version="0.7.9"

# set up functions to report Usage and Usage with Description
# config_path="/etc/conkywx"
bpath1="$( dirname "$0" )"
cwxpath="$( cd "$bpath1" && pwd )"
ProgName="$(basename $0)"
# dir1=${cwxpath/*\/}



usage1 ()
{
	sed >&2 -n '/^###/q;  /^#/!q;  s/^#//;  s/^ //;  4,$p' "$cwxpath/$ProgName"
}

# function to report error messages
infomsg ()
{
	# echo ""
	echo  >&2 "$1"
#	echo ""
	usage1
	exit 0
}

infomsg_stdout ()
{
	echo " " >&2
	echo -e "$1" >&2
	echo " " >&2
}



var_messages ()
{
#--------------------------------------------------------------------------
net_conn_find_msg="conkywx is checking for Internet connectivity"

#--------------------------------------------------------------------------
net_conn_found_msg="conkywx found Internet connectivity"

#--------------------------------------------------------------------------
net_conn_notfound_msg="conkywx found no Internet connectivity!
Trying to show last data update information."

#--------------------------------------------------------------------------
wx_update_ok="Weather update download completed."

#--------------------------------------------------------------------------
fupdate_msg="Force update completed for config file:
*$config_file*
conkywx weather data has been updated. 
Run your conky script to see updates on Desktop."

#--------------------------------------------------------------------------
fupdate_no_msg="Force update FAILED !!!"

#--------------------------------------------------------------------------
critical_warning_hdr="
*************** CRITICAL WARNING ***************"

#--------------------------------------------------------------------------
warning_hdr="
******************* WARNING *******************"

#--------------------------------------------------------------------------
resolve_msg="
Check if INTERNET connection is OK.
To resolve the issue:
type conkywx in a Terminal and choose option 3 FORCE update
OR try using another weather station.
OR if it looks fine type conkywx in a Terminal and choose option 4 get Debug 
   information"

#--------------------------------------------------------------------------
missing_template_msg=" $critical_warning_hdr

Unable to read template file.
Please check the template file - check the spellings
OR
$resolve_msg
$critical_warning_hdr"

#--------------------------------------------------------------------------
warning_msg_no_download="
DATA Error - unable to download data.
Please try again to connect once the system is online."

#--------------------------------------------------------------------------
warning_msg_no_bkup_data="
DATA Error - No INTERNET Connection.
Please try again to connect once the connection is online."

#--------------------------------------------------------------------------
warning_msg_both="
DATA Error - Check URL/ID for *Location* and *xml* in config file!

Possible errors could be:
    - Poor INTERNET connection.
    - Wrong xml_ID used.
    - Wrong Location used. 
Match the Location and xml website URL or ID correctly.
Make the required changes in your conkywx config file:
$config_file
$resolve_msg"

#--------------------------------------------------------------------------
warning_msg_location="
DATA Error - Check URL for *Location* settings in config file!

Possible errors could be:
    - Poor INTERNET connection.
    - Wrong Location used. 
Match the Location website URL or ID correctly.
Make the required changes in your conkywx config file:
$config_file
$resolve_msg"

#--------------------------------------------------------------------------
warning_msg_xml="
DATA Error - Some of the xml data is not correct!

Possible errors could be:
    - Poor INTERNET connection.
    - Wrong xml_ID used.
$resolve_msg"


#--------------------------------------------------------------------------
warning_msg1="

To get more information:
type conkywx in a Terminal and choose option 2 to view wiki."

#--------------------------------------------------------------------------
warning_msg_unknown="

Unhandled exception!!!
$resolve_msg

"

#--------------------------------------------------------------------------
debug_msg="**********************************************************

This file has all the data information that is used by the program. 

The file is divided in sections and each section 
has source information. This source has Location data or 
xml data. 

If *source location data* sections are empty then 
problem is with the Location ID or URL. 

If *source xml data* sections have empty information 
eg. all elements have no info then problem is with the xml_ID. 

The best way is to check the actual website if that has information. 

Sometimes other problems may occur which we have no knowledge of yet 
- this file will pinpout where the data is not coming out correct 
- could also be that the element used in the *conkyrc.sh file by 
the user may be in error.
 
**********************************************************

"

#--------------------------------------------------------------------------
new_ver_msg="
New conkywx version is available"

# ---------------------------------------------
no_config_file="$critical_warning_hdr

The specified conkywx config file: 
*$config_file*
does not exist...Exiting conkywx...
$critical_warning_hdr"

#--------------------------------------------------------------------------
showtime_err="
$warning_hdr
Check the template_cleanup option in your conkywx config file.

Make sure each word group has a colon *:* character separating the
left and the right side. Left side:Right side
Left side is what you want to change. 
Right side is the change.
$warning_hdr"

#--------------------------------------------------------------------------
readme_msg="
DO NOT EDIT ANY FILE IN THIS DIRECTORY
FILES IN THIS DIRECTORY ARE FOR INTERNAL CONKYWX USE ONLY
"

#--------------------------------------------------------------------------
uv_text="Low,Moderate,High,Very High,Extreme"

}

set_config ()
{
	# this dir holds the conkywx.conf user copy
	config_dir_path="$HOME/.config/conkywx"
	if [ ! -d $config_dir_path ]; then
		mkdir $config_dir_path
	fi
	# this dir holds program internal files which need to be persistant
	conkywx_internal_dir="$HOME/.config/conkywx/conkywx_internal"
	if [ ! -d $conkywx_internal_dir ]; then
		mkdir $conkywx_internal_dir
	fi
	if [ ! -s $conkywx_internal_dir/READ_ME ]; then
		echo "$readme_msg" > $conkywx_internal_dir/READ_ME
	fi
	
	if [[ -e "$config_file" ]]; then
		source "$config_file"
		# infomsg_stdout "$config_file"
	else
		infomsg_stdout "$no_config_file"
		exit 1
	fi
	
	conf1=${config_file##*/}
	temppath="/tmp/wx-${USER}-$conf1-pChA_r_27u0-m39"
	temppath_2="/tmp/wx-${USER}-$conf1-bkup"
	
	if [ ! -d $temppath ]; then
		mkdir $temppath
	fi
	if [ ! -d $temppath_2 ]; then
		mkdir $temppath_2
	fi
	
}

set_variables ()
{
	var_messages
	set_config
	# -----------------------------------------------------------------------
	# Data constants
	# -----------------------------------------------------------------------
	
	imagepath="$cwxpath/images"
	
	case $wx_iconset in
	1)
		wp1=01
		;;
	2)
		wp1=02
		;;
	3)
		wp1=03
		;;
	4)
		wp1=04
		;;
	*)
		wp1=01
		;;
	esac
	
	iwxpath="$imagepath/wxicons-$wp1"
	imoonpath="$imagepath/moonicons"
	
	tpath="$HOME/.conkywx-tpath"
	currdata="$temppath/currdata.html"
	currdata_bak="$temppath_2/currdata.html.bak"
	currdata_2="$temppath/currdata_2.xml"
	currdata_2_bak="$temppath_2/currdata_2.xml.bak"
	new_ver="$conkywx_internal_dir/conkywx_new_version_test"
	imp_cookies="$temppath/cookie"
	wund_debug_log="$temppath/wund-debug.log"
	nowCond="$temppath/nowCond.png"
	nowMoon="$temppath/nowmoon.png"
	
	dayinfo1="$temppath/dayinfo"
	today_forecast="$temppath/today_forecast"
	astro_forecast1="$temppath/astro_forecast"
	today_forecast_temp1="$temppath/today_forecast_temp1"
	today_xml_forecast1="$temppath/today_xml_forecast2.xml"
	viewtrends="$temppath/viewtrends"
	forecast_trends="$temppath/forecast_trends"
	forecast_trends_run="$temppath/forecast_trends_run"
	forecast_trends_time="$conkywx_internal_dir/forecast_trends_time-$conf1-${USER}"
	forecast_trends_time24="$conkywx_internal_dir/forecast_trends_time24-$conf1-${USER}"
	time_test="$temppath/time_test"
	data_check="$temppath/data_check"
	temp_template="$temppath/temp_template"
	wiki_location="https://bitbucket.org/plikhari/conkywx_pub/wiki/Home"
	
	# -----------------------------------------------------------------------
	# Data default settings
	# -----------------------------------------------------------------------
	# These variables have to be defined in the config file
	use_xml=${use_xml:?"Error: use_xml not defined in config file"}
	case $use_xml in
	0)
		xml_ID="Ignored_as_use_xml=0_in_config_file"
		;;
	1)
		xml_ID=${xml_ID:?"Error: xml_ID not defined in config file"}
		;;
	*)
		infomsg_stdout "Warning: Unknown option for *use_xml* in config file:\n"$config_file" \nxml_ID ignored. Showing only Location information."
		# exit 1
		;;
	esac
	
	Location=${Location:?"Error: Location not defined in config file."}

	Units_to_Use1=${Units_to_Use:?"Error: Units_to_Use not defined in config file."}
	Units_to_Use=${Units_to_Use1^^}
	if [[ "$Units_to_Use" != SI ]] && [[ "$Units_to_Use" != IM ]]; then
		infomsg_stdout "$critical_warning_hdr \nError: Units_to_Use: *$Units_to_Use1* is incorrect option in config file.\nValid values: SI [Metric] or IM [Imperial]."
		exit 1
	fi

	Units_downloaded1=${Units_downloaded:?"Error: Units_downloaded not defined in config file."}
	Units_downloaded=${Units_downloaded1^^}
	if [[ $Units_downloaded != SI ]] && [[ $Units_downloaded != IM ]]; then
		infomsg_stdout "$critical_warning_hdr \nError: Units_downloaded: *$Units_downloaded1* is incorrect option in config file.\nValid values: SI [Metric] or IM [Imperial]."
		exit 1
	fi
	
	# moon style - simple and outline 29 elements in desired sequence
	case $moon_outline in
	0)
		moon_font=( a b c c d e f g h i j k l m z y x x w v u t s r q p o n "0" "1" )
		;;
	1)
		#moon_font=( Z Y X X W V U T S R Q P O N M L K K J I H G F E D C B A "0" "1" )
		moon_font=( N O P P Q R S T U V W X Y Z A B C D E F G H I J K K L M "0" "1" )
		;;
	*)
		moon_font=( a b c c d e f g h i j k l m z y x x w v u t s r q p o n "0" "1" )
		;;
	esac
	
	# default values
	Default_Units_show=1
	Default_use_short_date=1
	Default_show_notification=1
	Default_Time_format=24
	Default_today_date_format="%A %b, %d %Y"
	Default_minimum_fetch_time=870
	Default_main_url="http://www.wunderground.com/global/stations/"
	Default_xml_url="http://api.wunderground.com/weatherstation/WXCurrentObXML.asp?ID="
	Default_net_connect="on"
	Default_show_lowercase_ampm=1
	Default_show_degsym_fc_temp=4

	# If these variables are NOT defined in config file - then default values are used
	Units_show=${Units_show:-$Default_Units_show}
	use_short_date=${use_short_date:-$Default_use_short_date}
	show_notification=${show_notification:-$Default_show_notification}
	Time_format=${Time_format:-$Default_Time_format}
	today_date_format=${today_date_format:-$Default_today_date_format}
	minimum_fetch_time=${minimum_fetch_time:-$Default_minimum_fetch_time}
	main_url=${main_url:-$Default_main_url}
	xml_url=${xml_url:-$Default_xml_url}
	net_connect=${net_connect:-$Default_net_connect}
	show_lowercase_ampm=${show_lowercase_ampm:-$Default_show_lowercase_ampm}
	show_degsym_fc_temp=${show_degsym_fc_temp:-$Default_show_degsym_fc_temp}
	
	
	case $use_xml in
	1)
		if [[ "$Location" =~ ^[0-9]+$ ]]; then
			curr_url="$main_url$Location.html"
			curr_url2=$xml_url$xml_ID
		else
			curr_url="$Location"
			curr_url2=$xml_url$xml_ID
		fi
		;;
	*)
		if [[ "$Location" =~ ^[0-9]+$ ]]; then
			curr_url="$main_url$Location.html"
			curr_url2=
		else
			curr_url="$Location"
			curr_url2=
		fi
		;;
	esac
}

# -----------------------------------------------------------------------
# show pop-up notifocation
# -----------------------------------------------------------------------
show_noti ()
{
	if [[ -e $nowCond ]]; then
		notiImage=$nowCond
	else
		notiImage=$iwxpath/pcrain.png
	fi
	
	if [[ $show_notification == "1" && "$(which notify-send)" != "" ]]; then
		notify-send "Conky Weather Update" \
			"$1" \
			-i $notiImage \
			-t 5000
	fi
	
}

# -----------------------------------------------------------------------
# check if connected to z internet
# -----------------------------------------------------------------------
check_net ()
{
	if [[ $( uptime | cut -c 14-15 ) -lt 1 ]]; then
		net_connect="on"
	else
		infomsg_stdout "$net_conn_find_msg"
		if eval "wget -q -O /dev/null --no-cache www.wunderground.com"; then
			net_connect="on"
			infomsg_stdout "$net_conn_found_msg"
		else
			net_connect="off"
			if [[ -s $currdata_bak ]]; then
				cp -p $currdata_bak $currdata
			elif [[ -s $currdata ]]; then
				cp -p $currdata $currdata_bak
			fi
			if [[ -s $currdata_2_bak ]]; then
				cp -p $currdata_2_bak $currdata_2
			elif [[ -s $currdata_2 ]]; then
				cp -p $currdata_2 $currdata_2_bak
			fi
			if [[ -z $debug ]]; then
				infomsg_stdout "$net_conn_notfound_msg" 
				show_noti "No INTERNET connectivity."
			fi
		fi
	fi
}

# -----------------------------------------------------------------------
# Get data from website
# -----------------------------------------------------------------------
get_data_file ()
{
	# DO NOT CHANGE THIS LINE !!!
	user_agent="Mozilla/4.74 [en] (X11; U; Linux 2.2.14 i686)"
	if [[ $net_connect == "on" ]]; then 
		wget --tries=99 --timeout=60 --user-agent="$user_agent" --save-cookies $imp_cookies -O $currdata $curr_url
		touch $currdata
		if [[ -s $currdata ]]; then
			cp -p $currdata $currdata_bak
		else
			infomsg_stdout "$warning_hdr $warning_msg_no_download $warning_hdr" &
			exit 0
		fi
		
		# if [[ $curr_url2 != "" ]]; then
		if [[ $use_xml -eq 1 ]]; then
			wget --tries=99 --timeout=60 --user-agent="$user_agent" --save-cookies $imp_cookies -O $currdata_2 $curr_url2
			touch $currdata_2
			if [[ -s $currdata_2 ]]; then
				cp -p $currdata_2 $currdata_2_bak
			else
				infomsg_stdout "$warning_hdr $warning_msg_no_download $warning_hdr" &
				exit 0
			fi
		fi
		
		if [[ "$enable_new_version_check" -eq 1 ]]; then
			t1=$( check_for_new_version )
		else
			t1=0
		fi
		
		# based on enable_new_version_check $t1 gives the wx_msg to show_noti
		case $t1 in
		0)
			wx_msg="$wx_update_ok"
			;;
		*)
			wx_msg="$wx_update_ok.
			$new_ver_msg $t1"
			;;
		esac

		infomsg_stdout "$wx_msg"
		show_noti "$wx_msg"
	else
		if [[ -s $currdata_bak ]]; then
			cp --preserve $currdata_bak $currdata
		fi
		if [[ -s $currdata_2_bak ]]; then
			cp --preserve $currdata_2_bak $currdata_2
		fi
	fi
}

# -----------------------------------------------------------------------
# check location n xml or minimum fetch time is adhered to
# -----------------------------------------------------------------------
check_for_data ()
{
#  checks for changes in location, use_xml and xml_ID - it will make a
#+ data call if any of them change
#  Then also checks for time - if the minimum_fetch_time interval exceeds - 
#+ it will make a data call
	do_gdf ()
	{
		check_net
		get_data_file
		echo "$Location|$use_xml|$xml_ID" > $data_check
	}
	
	if [[ -s $currdata ]]; then
		time_diff1=$(( nowtime1 - $( date -r $currdata +%s ) ))
	fi
	# if [[ -s $currdata_2 ]]; then
		# time_diff2=$(( nowtime1 - $( date -r $currdata_2 +%s ) ))
	# fi
	
	if [ ! -e $data_check ]; then touch $data_check; fi
	OLDIFS=$IFS; IFS=$'|'; set -f
	dcheck1=($(< "$data_check"))
	set +f; IFS=$OLDIFS

	check_location=${dcheck1[0]//[ ]/}
	check_xml=${dcheck1[1]//[ ]/}
	check_xml_ID=${dcheck1[2]//[ ]/}
	
	if [[ -z $fupdate ]]; then
		if [[ ! -s $currdata ]]; then
			do_gdf
		elif [[ "$Location" != "$check_location" ]] || [[ "$xml_ID" != "$check_xml_ID" ]] || [[ "$use_xml" != "$check_xml" ]]; then
			do_gdf
		elif [[ $time_diff1 -gt $minimum_fetch_time ]]; then
			do_gdf
		fi
	else
		do_gdf
		if [[ $net_connect == "on" ]]; then
			infomsg_stdout "$fupdate_msg"
		else
			infomsg_stdout "$fupdate_no_msg"
		fi
		exit 0
	fi
}

# -----------------------------------------------------------------------
# remove leading and trailing spaces n extra spaces bewteen words
# -----------------------------------------------------------------------
trim ()
{
	local var=("$1")
	echo ${var[@]}
}

# -----------------------------------------------------------------------
# convert forecast temps
# -----------------------------------------------------------------------
convert_FC_units ()
{
	if [[ $presenttemp == "N/A" ]]; then
		echo "N/A"
	else
		# change "23 | 21" temp format to "23/21" for day forecast
		infc1="$1"
		infc2=${infc1//[ ]/}   	# get rid of ALL spaces
		infcT=${infc2/\|*}		# get value left of |
		infcL=${infc2#*\|}		# get value right of |
		
		case $show_degsym_fc_temp in
		0)
			degsym=1
			;;
		1)
			degsym=2
			;;
		2)
			degsym=
			;;
		3)
			degsym=1
			degsym_at_end=1
			;;
		4)
			degsym=1
			degsym_at_end=2
			;;
		*)
			degsym=1
			degsym_at_end=2
			;;
		esac
		
		case $Units_to_Use in
		IM|MI)
			# 3rd option "1" is no Deg symbol "2" is with symble, 
			# remove 3rd option to get deg with F or C
			outfcT="$( convert_units "$infcT" "temp" $degsym )"
			outfcL="$( convert_units "$infcL" "temp" $degsym )"
			case $degsym_at_end in
			1)
				echo "$outfcT/$outfcL°"
				;;
			2)
				echo "$outfcT/$outfcL°F"
				;;
			*)
				echo "$outfcT/$outfcL"
				;;
			esac
			;;
		SI|IS)
			# 3rd option "1" is no Deg symbol "2" is with symble, 
			# remove 3rd option to get deg with F or C
			outfcT="$( convert_units "$infcT" "temp" $degsym )"
			outfcL="$( convert_units "$infcL" "temp" $degsym )"
			case $degsym_at_end in
			1)
				echo "$outfcT/$outfcL°"
				;;
			2)
				echo "$outfcT/$outfcL°C"
				;;
			*)
				echo "$outfcT/$outfcL"
				;;
			esac
			;;
		esac
	fi
}


# -----------------------------------------------------------------------
# unit conversions
# -----------------------------------------------------------------------
convert_units ()
{
	# $1 is unit_to_convert, $2 is unit_type (t,d,s,p), $3 is variable
	local unit_to_convert="$1"
	local unit_type=$2	# time, speed, distance, pressure
	# local unit_distance=$3	# long [km miles] or small [in or mm]
	# if [[ $unit_to_convert == "N/A" ]]; then
		# echo $unit_to_convert
	case $unit_to_convert in
	"N/A"|"-"|"")
		echo "N/A"
		;;
	*)
		case $Units_downloaded in
		SI|IS)
			case $Units_to_Use in
			IM|MI)
				# convert ditance, speed, temp, pressure
				case $unit_type in
				temp)
					# C to F
					outvalue1=$(echo $unit_to_convert | awk '{ printf( "%3.0f\n" ,((($1*9)/5)+32)) }')
					case $Units_show in 
					1)
						case $3 in
						1) # no_deg_sim
							outvalue="$outvalue1"
							;;
						2)
							outvalue="$outvalue1°"
							;;
						*)
							outvalue="$outvalue1°F"
							;;
						esac
						;;
					*)
						case $3 in
						1)
							outvalue="$outvalue1"
							;;
						*)
							outvalue="$outvalue1°"
							;;
						esac
						;;
					esac
					;;
				speed)
					case $3 in
					1)
						# km/hr to miles/hr
						outvalue1=$(echo $unit_to_convert | awk '{ printf( "%3.1f\n" ,($1*0.62137119)) }')
						case $Units_show in 
						1)
							outvalue="$outvalue1 mph"
							;;
						*)
							outvalue="$outvalue1"
							;;
						esac
						;;
					2)
						# km/hr to m/s
						outvalue1=$(echo $unit_to_convert | awk '{ printf( "%3.1f\n" ,($1*0.2777777778)) }')
						case $Units_show in 
						1)
							outvalue="$outvalue1 mps"
							;;
						*)
							outvalue="$outvalue1"
							;;
						esac
						;;
					esac
					;;
				distance)
					case $3 in
					3)
						# km to miles use $3 as indicator
						outvalue1=$(echo $unit_to_convert | awk '{ printf( "%3.1f\n" ,($1*0.62137119)) }')
						case $Units_show in 
						1)
							outvalue="$outvalue1 mi"
							;;
						*)
							outvalue="$outvalue1"
							;;
						esac
						;;
					4)
						# mm to inches $3 as indicator
						outvalue1=$(echo $unit_to_convert | awk '{ printf( "%3.1f\n" ,($1*0.039370078)) }')
						case $Units_show in 
						1)
							outvalue="$outvalue1 in"
							;;
						*)
							outvalue="$outvalue1"
							;;
						esac
						;;
					esac
					;;
				pressure)
					# hPa|millibars|mb to inches
					outvalue1=$(echo $unit_to_convert | awk '{ printf( "%3.2f\n" ,($1*0.02953)) }')
					case $Units_show in 
					1)
						outvalue="$outvalue1 in"
						;;
					*)
						outvalue="$outvalue1"
						;;
					esac
					;;
				esac
				;;
			SI|IS)
				case $unit_type in
				temp)
					outvalue1=$(echo $unit_to_convert | awk '{ printf("%3.0f\n",$1)}')
					case $Units_show in 
					1)
						case $3 in
						1) # no_deg_sim
							outvalue="$outvalue1"
							;;
						2)
							outvalue="$outvalue1°"
							;;
						*)
							outvalue="$outvalue1°C"
							;;
						esac
						;;
					*)
						case $3 in
						1)
							outvalue="$outvalue1"
							;;
						*)
							outvalue="$outvalue1°"
							;;
						esac
						;;
					esac
					;;
				speed)
					case $3 in
					1|2)
						case $Units_show in 
						1)
							outvalue="$unit_to_convert km/h"
							;;
						*)
							outvalue="$unit_to_convert"
							;;
						esac
						;;
					esac
					;;
				distance)
					case $3 in
					3)
						case $Units_show in 
						1)
							outvalue="$unit_to_convert km"
							;;
						*)
							outvalue="$unit_to_convert"
							;;
						esac
						;;
					4)
						case $Units_show in 
						1)
							outvalue="$unit_to_convert mm"
							;;
						*)
							outvalue="$unit_to_convert"
							;;
						esac
						;;
					esac
					;;
				pressure)
					case $Units_show in 
					1)
						outvalue="$unit_to_convert hPa"
						;;
					*)
						outvalue="$unit_to_convert"
						;;
					esac
					;;
				esac
				;;
			esac
			;;
		IM|MI)
			case $Units_to_Use in
			SI|IS)
				# convert ditance, speed, temp, pressure
				case $unit_type in
				temp)
					# F to C
					outvalue1=$(echo $unit_to_convert | awk '{ printf( "%3.0f\n" ,(($1-32)*(5/9))) }')
					case $Units_show in 
					1)
						case $3 in
						1)
							outvalue="$outvalue1"
							;;
						2)
							outvalue="$outvalue1°"
							;;
						*)
							outvalue="$outvalue1°C"
							;;
						esac
						;;
					*)
						case $3 in
						1)
							outvalue="$outvalue1"
							;;
						*)
							outvalue="$outvalue1°"
							;;
						esac
						;;
					esac
					;;
				speed)
					case $3 in
					1)
						# miles/hr to km/hr 
						outvalue1=$(echo $unit_to_convert | awk '{ printf( "%3.1f\n" ,($1/0.62137119)) }')
						case $Units_show in 
						1)
							outvalue="$outvalue1 km/h"
							;;
						*)
							outvalue="$outvalue1"
							;;
						esac
						;;
					2)
						#  m/s to km/hr
						outvalue1=$(echo $unit_to_convert | awk '{ printf( "%3.1f\n" ,($1/0.2777777778)) }')
						case $Units_show in 
						1)
							outvalue="$outvalue1 km/h"
							;;
						*)
							outvalue="$outvalue1"
							;;
						esac
						;;
					esac
					;;
				distance)
					case $3 in
					3)
						# miles to  km use $3 as indicator
						outvalue1=$(echo $unit_to_convert | awk '{ printf( "%3.1f\n" ,($1/0.62137119)) }')
						case $Units_show in 
						1)
							outvalue="$outvalue1 km"
							;;
						*)
							outvalue="$outvalue1"
							;;
						esac
						;;
					4)
						# inches to mm $3 as indicator
						outvalue1=$(echo $unit_to_convert | awk '{ printf( "%3.1f\n" ,($1/0.039370078)) }')
						case $Units_show in 
						1)
							outvalue="$outvalue1 mm"
							;;
						*)
							outvalue="$outvalue1"
							;;
						esac
						;;
					esac
					;;
				pressure)
					#  inches to  hPa|millibars|mb
					outvalue1=$(echo $unit_to_convert | awk '{ printf( "%3.2f\n" ,($1/0.02953)) }')
					case $Units_show in 
					1)
						outvalue="$outvalue1 hPa"
						;;
					*)
						outvalue="$outvalue1"
						;;
					esac
					;;
				esac
				;;
			IM|MI)
				case $unit_type in
				temp)
					outvalue1=$(echo $unit_to_convert | awk '{ printf("%3.0f\n",$1)}')
					case $Units_show in 
					1)
						case $3 in
						1)
							outvalue="$outvalue1"
							;;
						2)
							outvalue="$outvalue1°"
							;;
						*)
							outvalue="$outvalue1°F"
							;;
						esac
						;;
					*)
						case $3 in
						1)
							outvalue="$outvalue1"
							;;
						*)
							outvalue="$outvalue1°"
							;;
						esac
						;;
					esac
					;;
				speed)
					case $3 in
					1|2)
						case $Units_show in 
						1)
							outvalue="$unit_to_convert mph"
							;;
						*)
							outvalue="$unit_to_convert"
							;;
						esac
						;;
					esac
					;;
				distance)
					case $3 in
					3)
						case $Units_show in 
						1)
							outvalue="$unit_to_convert mi"
							;;
						*)
							outvalue="$unit_to_convert"
							;;
						esac
						;;
					4)
						case $Units_show in 
						1)
							outvalue="$unit_to_convert in"
							;;
						*)
							outvalue="$unit_to_convert"
							;;
						esac
						;;
					esac
					;;
				pressure)
					case $Units_show in 
					1)
						outvalue="$unit_to_convert in"
						;;
					*)
						outvalue="$unit_to_convert"
						;;
					esac
					;;
				esac
				;;
			esac
			;;
		esac
		echo $outvalue
		;;
	esac
}

# -----------------------------------------------------------------------
# convert times
# -----------------------------------------------------------------------
converttohours ()
{
	case $1 in
	""|"N/A"|"-")
		echo "N/A"
		;;
	*)
		local orgtime1=$( trim "$1" )
		local check_if_time="${orgtime1//[ ]*/}"
		local lt1=${check_if_time/\:*}		# get value left of :
		local rt1=${check_if_time#*\:}		# get value right of :
		
		if [[ $check_if_time =~ ^[0-9|:][:0-9]*$ && $((10#$lt1)) -le 24 && $((10#$rt1)) -le 60 ]]; then
			local corrtime="${orgtime1:0:8}"
			
			case $Time_format in
			24)
				t_format='+%R'
				;;
			12)
				case $show_lowercase_ampm in
				0) # show upper case
					t_format='+%I:%M %p'
					;;
				1) # show lower case
					t_format='+%I:%M %P'
					;;
				2) # no am pm displayed
					t_format='+%I:%M'
					;;
				3) # no leading zero
					t_format='+%_I:%M %P'
					;;
				4) # custom format
					t_format="$custom_time_format"
					;;
				*)
					t_format='+%I:%M %P'
					;;
				esac
				;;
			*)
				t_format='+%R'
				;;
			esac
			echo "$( date --date="$corrtime" "$t_format" )"
		else
			echo "N/A"
		fi
		;;
	esac
}

# -----------------------------------------------------------------------
# convert times used by get_state_image
# -----------------------------------------------------------------------
do_hour_check ()
{
	case $1 in
	"N/A"|""|"-")
		echo "N/A"
		;;
	*)
		local o_time="$1"
		local corrtime1="${o_time:0:8}"
		
		if [[ $Time_format == 12 ]]; then
			echo $( date --date="$corrtime1" +%R )
		else
			echo "$1"
		fi	
		;;
	esac
	
}


# -----------------------------------------------------------------------
# function for testing speed of execution
# -----------------------------------------------------------------------
script_time ()
{
	# nowtime2=$( date +%s )
	nowtime2=$( date +%s%N )
	time_taken=$(( nowtime2 - nowtime_testing ))
	echo "time taken $time_taken sec"
}

# -----------------------------------------------------------------------
# get forecast trends and arrow fonts
# -----------------------------------------------------------------------
get_forecast_trends ()
{
	#+ There are 2 type of trends 
	#+ - Type 1 that are updated hourly
	#+ - pressure, presenttemp, windspeed, humidity, UV, dewpoint, visibility.
	#+ - Type 2 that are updated every 24 hours
	#+ - Sunrise, Sunset, Moonrise, Moonset, length of Vis light, length of day, Moon Illumination.
	
	#+ You will not see trends for an element if the value of the Current data or the value of the stored data is N/A. This is because there is no way to get a comparison when one of the data is Not Available.
	
	# Note: Please do not expect to see many trend changes if you are using Location - unless it is Rapid Fire type. If you see steady arrow <> for most of the type 1 trends - check the Last Update time - which will give you the reason. For example the last update I got was 8:30 pm and it is 10 pm now and all the type 1 trends are steady. To get faster performance - in the config file - use the use_xml=1 option and select appropriate xml_ID.
if [[ $presenttemp == "N/A" ]]; then
	trends_out=( - - - - - - - - - - - - - - - - - - - - - - - - - - - - - )
	echo ${trends_out[@]}
	echo >&2 "No data trends as present temp N/A"
else
	tformat=$Time_format; Time_format=24
	srise1=$( converttohours "${astro_forecast[4]}" )
	sset1=$( converttohours "${astro_forecast[5]}" )
	mrise1=$( converttohours "${astro_forecast[16]}" )
	mset1=$( converttohours "${astro_forecast[17]}" )
	Time_format=$tformat
	# dt=$( date --date="${astro_forecast[0]}" '+%d' )
	
	# present data
	# rapid changes 30 min
	ft_data=( $pressure $presenttemp $windspeed_trend $humidity $uv2 $dewpoint $visibility )
	# changes 24 hours
	ft_time=( $srise1 $sset1 $mrise1 $mset1 "$lvlight" "$lofday" $percentilluminated )
	ft2_time=( $( trim "${ft_time[*]//[.:h mAMPamp]/}" ) )
	
	# store data if empty or fresh start
	if [ ! -e $forecast_trends ]; then 
		echo "${ft_data[*]}" > $forecast_trends
	fi
	if [ ! -e $forecast_trends_run ]; then 
		echo "${ft_data[*]}" > $forecast_trends_run
	fi
	if [ ! -e $forecast_trends_time ]; then 
		echo "${ft2_time[*]}" > $forecast_trends_time 
	fi
	if [ ! -e $forecast_trends_time24 ]; then 
		echo "${ft2_time[*]}" > $forecast_trends_time24
	fi
	
	# read stored data
	OLDIFS=$IFS; IFS=$' '; set -f
	local storeddata=($(< "$forecast_trends_run"))
	local storeddata_time=($(< "$forecast_trends_time24"))
	local storeddata_time_now=($(< "$forecast_trends_time"))
	set +f; IFS=$OLDIFS
	
	# Consolidate present and stored data
	nowdata=( ${ft_data[@]} ${ft2_time[@]} )
	nowstored=( ${storeddata[@]} ${storeddata_time[@]} )

	# test data
	# ft2_time=(  0630 1739 1639 0501 1157 1108 100 )
	# storeddata_time=(  0630 1739 1639 0501 1157 1108 98 )
	# nowdata=( 1011.1 25.8 0 59 0 17.2 2.1 0630 1739 1639 0501 1157 1108 100 )
	# nowstored=( 1011.2 26.2 0 57 0.0 17.1 2.2 0630 1739 1639 0501 1157 1108 98 )

	local file_create_time1=$( date -r $forecast_trends +%s )
	local file_create_time2=$( date -r $forecast_trends_time +%s )
	local file_create_time3=$( date -r $forecast_trends_time24 +%s )
	local time_diff2=$(( nowtime1 - file_create_time1 ))
	local time_diff_time=$(( nowtime1 - file_create_time2 ))
	local time_diff_bet_time=$(( file_create_time3 - file_create_time2 ))
	time_diff2=${time_diff2//-/}
	time_diff_time=${time_diff_time//-/}
	time_diff_bet_time=${time_diff_bet_time//-/}
	
	# set -x
	for ((l=0; l<14; l++));
		do
			local p1=$( check_num "${nowdata[$l]}" )
			local s1=$( check_num "${nowstored[$l]}" )
			
			if [[ "$p1" == "N/A" || "$s1" == "N/A" ]]; then
				trend1="-"
				trend_arrow1="-"
			else
				toption=$( echo "$p1" "$s1" | awk '{
					if ( $1 == $2 ) t_text=1;
					else if ( $1 < $2 ) t_text=2;
					else if ( $1 > $2 ) t_text=3;
					else t_text="0";
					print t_text;
					}' )
				if [[ $toption == "1" ]]; then
					trend1="Steady"
					trend_arrow1=$( echo -e $steadytrend )
				elif [[ $toption == "2" ]]; then
					if [[ $l -gt 6 ]] && [[ $l -lt 11 ]]; then
						trend1="Ealier"
					else
						trend1="Falling"
					fi
					trend_arrow1=$( echo -e $fallingtrend )
				elif [[ $toption == "3" ]]; then
					if [[ $l -gt 6 ]] && [[ $l -lt 11 ]]; then
						trend1="Later"
					else
						trend1="Rising"
					fi
					trend_arrow1=$( echo -e $risingtrend )
				elif [[ $toption == "0" ]]; then
					trend1="-"
					trend_arrow1="-"
				fi
			fi
		
			trend+=( $trend1 )
			trend_arrow+=( $trend_arrow1 )
		done
	
	# trend_out - total trend array output
	trends_out=( "${trend[@]}" "${trend_arrow[@]}" )
	
	# to see trend table in terminal with each update to test
	echo "
	 ------------------------------------------------- 
	 Forecast trend Hourly and Daily data table
	 ------------------------------------------------- 
	 Weather Element : Press  Temp Wind Hum UV Dew  Vis
	 Hourly present  : ${ft_data[*]}
	 Hourly stored   : ${storeddata[*]}
	 Hourly out Text : ${trend[*]:0:7}
	 Hourly out Font : ${trend_arrow[*]:0:7}
	 ------------------------------------------------- 
	 Weather Element : SunR SunS MoonR MoonS LVL LOD MoonIll
	 Daily  present  : ${ft2_time[*]}
	 Daily  stored   : ${storeddata_time[*]}
	 Daily  out Text : ${trend[*]:7:7}
	 Daily  out Font : ${trend_arrow[*]:7:7}
	 -------------------------------------------------
	 "  > $viewtrends
	
	#+ uncomment this line - One can view this table in *conkywx assistant* as well
	# cat $viewtrends >&2
	
	# trend keys
	# text :: $pressure 0 $presenttemp 1 $windspeed 2 $humidity 3 $uv2 4 $dewpoint 5 $visibility 6 $srise1 7 $sset1 8 $mrise1 9 $mset1 10 "$lvlight" 11 "$lofday" 12 $percentilluminated 13
	# fonts :: $pressure 14 $presenttemp 15 $windspeed 16 $humidity 17 $uv2 18 $dewpoint 19 $visibility 20 $srise1 21 $sset1 22 $mrise1 23 $mset1 24 "$lvlight" 25 "$lofday" 26 $percentilluminated 27
	echo ${trends_out[@]}
	
	# write every half hour
	if [[ $time_diff2 -gt 1800 ]]; then
		cp -p $forecast_trends $forecast_trends_run
		echo "${ft_data[*]}" > $forecast_trends
	fi
	# write every 24 hour
	if [[ $time_diff_time -gt 86400 ]]; then
		cp -p $forecast_trends_time $forecast_trends_time24
		echo "${ft2_time[*]}" > $forecast_trends_time
	fi
	# if [[ $time_diff_bet_time -gt 86400 ]]; then
		# cp -p $forecast_trends_time $forecast_trends_time24
	# fi
	
	# handle moon reaching 0 or 100 % illumination
	# current nowdata is consolidated array for ( ${ft_data[@]} ${ft2_time[@]} )
	# filed nowstored is consolidated array for ( ${storeddata[@]} ${storeddata_time[@]} )
	# in these array element 13 is for moon ill
	# these array are mind benders lol
	if [[ ${nowdata[13]} -eq 100 ]]; then
		if [[ ${nowstored[13]} -ne 100 ]]; then
			
			# create temp files to store both trend file timestamps
			TMPFILE24="$(mktemp)"
			TMPFILE1="$(mktemp)"
			# save the timestamps
			touch -r $forecast_trends_time24 $TMPFILE24
			touch -r $forecast_trends_time $TMPFILE1
			
			# moon element is position 6 [ arrays start with num 0 ]
			pos=6
			
			# make changes to 24 hour old data
			# completely remove the moon element
			storeddata_time=( ${storeddata_time[@]:0:$pos} ${storeddata_time[@]:$(($pos + 1))} )
			# add the updated moon element to array
			storeddata_time=( "${storeddata_time[@]}" "${ft2_time[$pos]}" )
			
			# make changes to current data
			# completely remove the moon element
			storeddata_time_now=( ${storeddata_time_now[@]:0:$pos} ${storeddata_time_now[@]:$(($pos + 1))} )
			# add the updated moon element to array
			storeddata_time_now=( "${storeddata_time_now[@]}" "${ft2_time[$pos]}" )
			
			# write updated original data with updated moon data
			echo "${storeddata_time[*]}" > $forecast_trends_time24
			echo "${storeddata_time_now[*]}" > $forecast_trends_time
			
			# restore the timestamp
			touch -r $TMPFILE24 $forecast_trends_time24
			touch -r $TMPFILE1 $forecast_trends_time
			
			# delete the temp files used
			rm $TMPFILE24 $TMPFILE1
		fi
	elif [[ ${nowdata[13]} -eq 0 ]] ; then
		if [[ ${nowstored[13]} -ne 0 ]]; then
			
			# create temp files to store both trend file timestamps
			TMPFILE24="$(mktemp)"
			TMPFILE1="$(mktemp)"
			# save the timestamps
			touch -r $forecast_trends_time24 $TMPFILE24
			touch -r $forecast_trends_time $TMPFILE1
			
			# moon element is position 6 [ arrays start with num 0 ]
			pos=6
			
			# make changes to 24 hour old data
			# completely remove the moon element
			storeddata_time=( ${storeddata_time[@]:0:$pos} ${storeddata_time[@]:$(($pos + 1))} )
			# add the updated moon element to array
			storeddata_time=( "${storeddata_time[@]}" "${ft2_time[$pos]}" )
			
			# make changes to current data
			# completely remove the moon element
			storeddata_time_now=( ${storeddata_time_now[@]:0:$pos} ${storeddata_time_now[@]:$(($pos + 1))} )
			# add the updated moon element to array
			storeddata_time_now=( "${storeddata_time_now[@]}" "${ft2_time[$pos]}" )
			
			# write updated original data with updated moon data
			echo "${storeddata_time[*]}" > $forecast_trends_time24
			echo "${storeddata_time_now[*]}" > $forecast_trends_time
			
			# restore the timestamp
			touch -r $TMPFILE24 $forecast_trends_time24
			touch -r $TMPFILE1 $forecast_trends_time
			
			# delete the temp files used
			rm $TMPFILE24 $TMPFILE1
		fi
	fi
fi
}

# -----------------------------------------------------------------------
# convert long day names to shorter 3 character days
# -----------------------------------------------------------------------
get_short_date ()
{
	local strg1=$( trim "$1" )
	case $use_short_date in
	1)
		echo "${strg1:0:3} ${strg1#*\ }"
		;;
	*)
		echo $strg1
		;;
	esac
}

# -----------------------------------------------------------------------
# test if variable is positive
# -----------------------------------------------------------------------
test_positive ()
{
	local test_val="$1"
	if [[ ${test_val:0:1} == "-" ]]; then
		echo "minus"
	else
		echo "plus"
	fi
}


# -----------------------------------------------------------------------
# figure out the how well cooked will one get - UV text ;-)
# -----------------------------------------------------------------------
get_uv_index_text ()
{
	# set -x
	case $1 in
	"N/A"|""|"-")
		echo "N/A"
		;;
	*)
		if [[ $( test_positive $1 ) == "plus" ]]; then
			uvin1="$1"
			uv_text1=$( echo $uvin1 | awk -v "uv_text1=${uv_text}" '{
				uv_index=$1; split(uv_text1,uv_text,",");
				if ( uv_index <= 0 ) uv_text2="N/A";
				else if ( uv_index <= 2 ) uv_text2=uv_text[1];
				else if ( uv_index <= 5 ) uv_text2=uv_text[2];
				else if ( uv_index <= 7 ) uv_text2=uv_text[3];
				else if ( uv_index < 11 ) uv_text2=uv_text[4];
				else if ( uv_index >= 11 ) uv_text2=uv_text[5];
				else uv_text2="N/A";
				print uv_text2;
				}' )
		else
			uv_text1="N/A"
		fi
		echo "$uv_text1"
	esac
	# set +x
}

# -----------------------------------------------------------------------
# Calculate the feel good / heat index temp
# -----------------------------------------------------------------------
get_apparent_temp ()
{
	case $presenttemp in
	"N/A"|""|"-")
		app_temp="N/A"
		;;
	*)
		case $Units_downloaded in
		IM|MI)
			# temp F to C
			presenttemp1=$(echo $presenttemp | awk '{ printf( "%3.1f\n" ,(($1-32)*(5/9))) }')
			# speed mph to km/hr
			windspd=$(echo $windspeed | awk '{ printf( "%3.1f\n" ,($1/0.62137119)) }')
			;;
		IS|SI)
			presenttemp1="$presenttemp"
			windspd=$windspeed
			;;
		esac
		
		app_temp1=$( echo $presenttemp1 $humidity $windspd | awk '{ 
			watervap_pressure=( $2 / 100 * 6.105 * 2.718281828 ^ (( 17.27 * $1 ) / ( 237.7 + $1 )));
			app_temp2=($1+(0.33*watervap_pressure) - (0.7*($3*0.27777778)) - 4.00 );
			printf( "%3.1f\n", app_temp2 );
			}' )
			
		# convert temp to appropriate units - units in C from above forluma
		case $Units_to_Use in
		IM|MI)
			Units_to_Use_org=$Units_to_Use; Units_to_Use=IM
			Units_downloaded_org=$Units_downloaded; Units_downloaded=SI
			app_temp="$( convert_units "$app_temp1" "temp" )"
			Units_to_Use=$Units_to_Use_org; Units_downloaded=$Units_downloaded_org
			;;
		IS|SI)
			Units_to_Use_org=$Units_to_Use; Units_to_Use=SI
			Units_downloaded_org=$Units_downloaded; Units_downloaded=SI
			app_temp="$( convert_units "$app_temp1" "temp" )"
			Units_to_Use=$Units_to_Use_org; Units_downloaded=$Units_downloaded_org
			;;
		esac
		;;
	esac
	
	echo "$app_temp"
}

# -----------------------------------------------------------------------
# check if value is number including float -ve etc
# -----------------------------------------------------------------------
check_num ()
{
	mynum1="$1"; mynum=${mynum1//[ ]/}
	case "$mynum" in
	"N/A"|""|"-")
		echo "N/A"
		;;
	*)
		if [[ "$mynum" =~ ^[-|+|0-9|.][.0-9]*$ ]]; then
			echo "$mynum"
		else
			echo "N/A"
		fi
		;;
	esac
	
}

# -----------------------------------------------------------------------
# check if xml and Location values are giving valid results
# -----------------------------------------------------------------------
check_status ()
{
if [[ -z $debug ]]; then
	case $net_connect in
	"on")
		case $use_xml in
		1)
			if [[ ! -s $today_forecast || $station_id == "" ]]; then
				if [[ $station_id == "" && ! -s $today_forecast ]]; then
					warning_msg="$warning_msg_both $warning_msg1"
				elif [[ ! -s $today_forecast ]]; then
					warning_msg="$warning_msg_location $warning_msg1"
				elif [[ $station_id == "" ]]; then
					warning_msg="$warning_msg_xml $warning_msg1"
				fi
			fi
			;;
		*)
			if [[ ! -s $today_forecast ]]; then
				warning_msg="$warning_msg_location $warning_msg1"
			fi
			;;
			
		esac
		if [[ ! -z $warning_msg ]]; then
			infomsg_stdout "$warning_hdr $warning_msg $warning_hdr" &
		fi
		;;
	*)
		if [[ ! -s $currdata_bak ]]; then
			infomsg_stdout "$warning_hdr $warning_msg_no_bkup_data $warning_hdr" &
		fi
		;;
	esac
fi
}

# -----------------------------------------------------------------------
# Extract data from downloaded files
# -----------------------------------------------------------------------
get_present_data ()
{
if [[ -s $currdata ]]; then
	sed -n '/layerCurrent/,/loadHistory/{s/<[^>]*>//g;:a;/</N;//ba;s/^[ \t]*//;s/[ \t]*$//;s/\&.*//g;/function/,/jQuery/d;/Elevation/,/Station Select/d;/Feels Like/,/Conditions/d;0,/Temperature/s/Temperature/Temp_now/;/speak METAR/,/Temperature/d; s/^Speed.*//;s/^\/.*//;s/ out of.*//;s/Updated /&\n/;/^$/d;p}' $currdata > $today_forecast
	
	quality_of_fcast=$(sed -n '/forecasted high temperature/{s/<[^>]*>//g;:a;/</N;//ba;s/^[ \t]*//;s/[ \t]*$//;s/.*is //;s/\..*//;p;q;}' $currdata)
	
	OIFS="$IFS"; IFS=$'\n'; set -f
	astro_forecast=($( sed -n '/astro_contain/,/Visit Astronomy/{s/<[^>]*>//g;:a;/</N;//ba;s/^[ \t]*//;s/[ \t]*$//;/^$/d;p}' $currdata ))
	# used by new moon and full moon as their location is variable in the data
	echo "${astro_forecast[*]}" > $astro_forecast1
	#### 7 day forecast
	dayinfo=($(sed -n '/viewCal/,/BestForecast/{s/<[^>]*>//g;:a;/</N;//ba;s/^[ \t]*//;s/[ \t]*$//;s/ \&.*//g;/Precipitation/d;/View Calendar/d;/BestForecast/d;/^$/d;p}' $currdata ))
	set +f; IFS="$OIFS"
	
	fetchtime=$( converttohours "$( date -r $currdata +%R )")
	srise=$( converttohours "${astro_forecast[4]}" )
	sset=$( converttohours "${astro_forecast[5]}" )
	statname=$( trim "$(sed -n '/^Updated/{n;n;n;p;}' $today_forecast)" )
	mrise=$( converttohours "${astro_forecast[16]}" )
	mset=$( converttohours "${astro_forecast[17]}" )
	lvlight=$( trim "${astro_forecast[19]}" )
	lofday=$( trim "${astro_forecast[21]}" )
	moonphase=$( trim "${astro_forecast[25]}" )
	presentcondition=$( trim "$(sed -n '/^Now$/{n;p;}' $today_forecast )" )
	metar1=$( trim "$(sed -n '/^METAR$/{n;p;}' $today_forecast )" )
	snowdepth=$( trim "$(sed -n '/^Snow/{n;s/ in//;p;}' $today_forecast )")
	# snowdepth="$( echo "$(sed -n '/^Snow/{n;p;}' $today_forecast )" | awk '{ printf( "%3.0f\n" ,($1/0.039370078)) }' )"
	# infomsg_stdout "$snowdepth"
	clouds=$( trim "$(sed -n '/^Clouds/{n;p;}' $today_forecast )" )
	if [[ $clouds == "Moisture" ]]; then clouds="-"; fi
	visibility=$( check_num $(sed -n '/^Visibility$/{n;p;}' $today_forecast ) )
	pollen=$(sed -n '/^Pollen$/{n;p;}' $today_forecast )
	moonp1="${moonphase/\ *}"
	moonp2="${moonphase#*\ }"
	if [[ "$moonp1" == "$moonp2" ]]; then moonp2="Moon"; fi
	pi1="${astro_forecast[23]#*\, }"; percentilluminated=$( check_num ${pi1/\%*} )
	newmoon=$( sed -n '/^New$/{g;1!p;};h' $astro_forecast1 | head -1 )
	fullmoon=$( sed -n '/^Full$/{g;1!p;};h' $astro_forecast1 | head -1 )
	
	if [[ $use_xml = "1" ]]; then
		OIFS="$IFS"; IFS=$'\n'; set -f
		today_xml_forecast=($(sed 's/^[ \t]*//;s/[ \t]*$//;/^$/d;/<\?xml/,/\/full/d;/<city/,/\/location/d;/weather/d; /heat_index/,/solar_radiation/d;/history_url/,/current_observation/d;s/<\/.*>//g;s/<//g; s/>/\n /g;s/^$/-/g' $currdata_2))
		set +f; IFS="$OIFS"
		
		station_id=$( trim "${today_xml_forecast[3]}" )
		stationname="${statname/\,*} | $station_id"
		lu2=$( converttohours "${today_xml_forecast[7]#*\, }" )
		last_update_time="$( converttohours "$( sed -n '/^Updated/{n;p;}' $today_forecast )" )"
		lastupdate="$last_update_time | $lu2 - $station_id"
		humidity=$( check_num "${today_xml_forecast[17]}" )
		uv2_xml=$( check_num ${today_xml_forecast[41]} )
		case $uv2_xml in
		""|"N/A"|"-")
			uv2=$( check_num $( sed -n '/^Health$/{n;n;s/ out of.*//;p;}' $today_forecast ))
			;;
		*)
			uv2=$uv2_xml
			;;
		esac
		
		winddir1=${today_xml_forecast[21]}
		
		case $Units_downloaded in
		IM|MI)
			presenttemp=$( check_num "${today_xml_forecast[13]}" )
			pressure=$( check_num "${today_xml_forecast[33]}" )
			rainfall=$( check_num "${today_xml_forecast[51]}" )
			windspeed1=$( check_num ${today_xml_forecast[25]} )
			windgust1=$( check_num ${today_xml_forecast[27]} )
			dewpoint=$( check_num ${today_xml_forecast[37]} )
			;;
		IS|SI)
			presenttemp=$( check_num "${today_xml_forecast[15]}" )
			pressure=$( check_num "${today_xml_forecast[31]}" )
			rainfall=$( echo $( check_num "${today_xml_forecast[53]:0:5}" ) | awk '{printf("%3.1f\n", $1*10)}')
			windspeed1=$(echo $( check_num ${today_xml_forecast[25]} ) | awk '{ printf( "%3.1f\n" ,($1/0.62137119)) }')
			windgust1=$(echo $( check_num ${today_xml_forecast[27]} ) | awk '{ printf( "%3.1f\n" ,($1/0.62137119)) }')
			dewpoint=$( check_num ${today_xml_forecast[39]} )
			;;
		esac
	else
		stationname="${statname/\,*}"
		lastupdate="$( converttohours "$( sed -n '/^Updated/{n;p;}' $today_forecast )")" 
		last_update_time=$lastupdate
		presenttemp=$( check_num $( sed -n '/^Temp_now$/{n;p;}' $today_forecast ))
		pressure=$( check_num $( sed -n '/^Pressure$/{n;p;}' $today_forecast ))
		humidity=$( check_num $( sed -n '/^Humidity$/{n;s/\%//p;}' $today_forecast ))
		rainfall=$( check_num $( sed -n '/^Rainfall$/{n;p;}' $today_forecast ))
		uv2=$( check_num $( sed -n '/^Health$/{n;n;s/ out of.*//;p;}' $today_forecast ))
		windspeed1=$( sed -n '/^Wind$/{n;p;}' $today_forecast )
		winddir1=$( trim "$( sed -n '/^from$/{n;p;}' $today_forecast )" )
		windgust1=$( check_num $( sed -n '/^Wind Gust$/{n;p;}' $today_forecast ))
		dewpoint=$( check_num $( sed -n '/^Dew Point$/{n;p;}' $today_forecast ))
	fi
	
	
	# handling wind
	case $windspeed1 in
	Calm|0|0.0)
		windspeed="Calm"
		windspeed_trend=0
			case $windgust1 in
			0|0.0)
				winddir=" N/A"
				windgust="N/A"
				;;
			*)
				winddir=" Var"
				windgust=$windgust1
				;;
			esac
		;;
	"N/A")
		windspeed="N/A"
		windspeed_trend="N/A"
		windgust="N/A"
		winddir=" N/A"
		;;
	*)
		windspeed=$( check_num $windspeed1 )
		winddir=$winddir1
		windspeed_trend=$windspeed
		windgust=$windgust1
		;;
	esac
	
	if [[ $uv2 == "N/A" ]]; then
		uvtext="N/A"
	else
		uvtext=$( get_uv_index_text "$uv2")
	fi
else
	lastupdate="N/A";srise="N/A";sset="N/A";statname="N/A";mrise="N/A";mset="N/A";
	lvlight="N/A";lofday="N/A";moonphase="N/A";presentcond="N/A";metar1="N/A";
	snowdepth="N/A";visibility="N/A";pollen="N/A";moonp1="N/A";moonp2="N/A";
	newmoon="N/A";fullmoon="N/A";presenttemp="N/A";pressure="N/A";rainfall="N/A";
	windspeed="N/A";windgust="N/A";dewpoint="N/A";winddir="N/A";humidity="N/A";
	uv2="N/A";uvtext="N/A";percentilluminated="N/A"
	
	echo >&2 "********************** CRITICAL WARNING ************************"
	echo >&2 "current data file is zero size - possibly no Internet connection"
	echo >&2 "****************************************************************"
fi

}

# -----------------------------------------------------------------------
# Figure out the character for each bearing
# -----------------------------------------------------------------------
get_bearing_font ()
{
	wind_bearing1=${winddir//[ ]/}
	wind_bearing="${wind_bearing1^^}"
	case $wind_bearing in
	N|NORTH)
		fnum=I
		;;
	NNE|"NORTH NORTH EAST")
		fnum=J
		;;
	NE|"NORTH EAST")
		fnum=K
		;;
	ENE|"EAST NORTH EAST")
		fnum=L
		;;
	E|EAST)
		fnum=M
		;;
	ESE|"EAST SOUTH EAST")
		fnum=N
		;;
	SE|"SOUTH EAST")
		fnum=O
		;;
	SSE|"SOUTH SOUTH EAST")
		fnum=P
		;;
	S|SOUTH)
		fnum=A
		;;
	SSW|"SOUTH SOUTH WEST")
		fnum=B
		;;
	SW|"SOUTH WEST")
		fnum=C
		;;
	WSW|"WEST SOUTH WEST")
		fnum=D
		;;
	W|WEST)
		fnum=E
		;;
	WNW|"WEST NORTH WEST")
		fnum=F
		;;
	NW|"NORTH WEST")
		fnum=G
		;;
	NNW|"NORTH NORTH WEST")
		fnum=H
		;;
	VARIABLE|VAR)
		fnum="\""
		;;
	"N/A"|NOT*|"-")
		fnum="-"
		;;
	*)
		fnum="%"
		;;
	esac
	
	echo $fnum
	
}

# -----------------------------------------------------------------------
# Process state of weather like blowing sand, partly cloudy etc
# -----------------------------------------------------------------------
get_dailyimage ()
{
	case $2 in
	""|"-"|"N/A")
		dailyimage="unknown"
		;;
	*)
		process_dm1="$2"
		process_dm2="${process_dm1,,}"
		process_dm=$( echo $process_dm2 | sed 's/light //;s/heavy //;s/severe //' )
		
		case $process_dm in
		chance*flurries)
			dailyimage=chanceflurries
			;;
		chance*rain)
			dailyimage=chancerain
			;;
		chance*freezing*rain|chance*sleet)
			dailyimage=chancesleet
			;;
		chance*snow)
			dailyimage=chancesnow
			;;
		chance*thunderstorm*|chance*t-storm*)
			dailyimage=chancetstorms
			;;
		"clear")
			dailyimage=clear
			;;
		cloudy|overcast)
			dailyimage=cloudy
			;;
		*flurries*)
			dailyimage=flurries
			;;
		*fog*|*mist*)
			dailyimage=fog
			;;
		*haz*)
			dailyimage=hazy
			;;
		*sand*|*dust*)
			dailyimage=dust
			;;
		*wind*)
			dailyimage=windy
			;;
		*ice*)
			dailyimage=ice
			;;
		smoke)
			dailyimage=smoke
			;;
		"mostly cloudy"|"partly sunny")
			dailyimage=mostlycloudy
			;;
		"partly cloudy"|"scattered clouds"|"mostly sunny")
			dailyimage=partlycloudy
			;;
		*freezing*[rain]*|*sleet*|*ice*shower*)
			dailyimage=sleet
			;;
		rain*|shower*|"drizzle" )
			dailyimage=rain
			;;
		*snow*)
			dailyimage=snow
			;;
		"sunny")
			dailyimage=sunny
			;;
		thunderstorm*|t-storm*)
			dailyimage=tstorms
			;;
		"unknown")
			dailyimage=unknown
			;;
		*)
			# get present condition if it matches none of the above 
			if [[ -s $currdata ]]; then
				oi_1=
				if [[ $3 == "" ]]; then
					dailyimage=$( sed -n '/curIcon/{s/.gif.*//;s/^.*\///p}' $currdata )
					oi_1=1
				fi
			fi
			;;
		esac
		;;
	esac
	
	get_state_image $1 $dailyimage $3
}

# -----------------------------------------------------------------------
# Get image or font character for weather state
# -----------------------------------------------------------------------
get_state_image ()
{
	corr_f_time=$( do_hour_check "$last_update_time" )
	corr_sset=$( do_hour_check "$sset" )
	corr_srise=$( do_hour_check "$srise" )
	
	processimage=$2
	icon1=$(echo "${processimage,,}")
	case ${lofday} in
	"N/A"|""|"-")
		imageout=na
		fontchar="-"
		;;
	*)
		if [[ ${lofday//[hH]*/} -ne 24 ]]; then
			if [[ $corr_f_time > $corr_sset || $corr_f_time < $corr_srise ]]; then
				if [[ $oi_1 -ne 1 ]]; then
					if [[ $1 -eq 1 ]]; then
						icon1=""nt_""$icon1""
					fi
				fi
			fi
		fi
	
		case $icon1 in
			chanceflurries|flurries)
				imageout=snow
				fontchar="p"
				;;
			snow)
				imageout=snow
				fontchar="q"
				;;
			chancerain)
				imageout=pcrain
				fontchar="h"
				;;
			chancesleet|chancesnow)
				imageout=pcsnow
				fontchar="o"
				;;
			chancetstorms)
				imageout=mctstorm
				fontchar="k"
				;;
			clear|sunny)
				imageout=sunny
				fontchar="a"
				;;
			cloudy)
				imageout=cloudy
				fontchar="e"
				;;
			fog|nt_fog)
				imageout=fog
				fontchar="9"
				;;
			hazy|nt_hazy)
				imageout=hazy
				fontchar="9"
				;;
			smoke|nt_smoke)
				imageout=smoke
				fontchar="9"
				;;
			windy|nt_windy)
				imageout=windy
				fontchar="6"
				;;
			dust|nt_dust)
				imageout=dust
				fontchar="9"
				;;
			ice|nt_ice)
				imageout=ice
				fontchar="w"
				;;
			mostlycloudy|partlysunny)
				imageout=mcloudy
				fontchar="d"
				;;
			fair)
				imageout=fair
				fontchar="b"
				;;
			partlycloudy|mostlysunny)
				imageout=pcloudy
				fontchar="c"
				;;
			sleet)
				imageout=sleet
				fontchar="w"
				;;
			rain)
				imageout=rain
				fontchar="i"
				;;
			tstorms)
				imageout=tstorm
				fontchar="n"
				;;
			nt_fair)
				imageout=ntfair
				fontchar="B"
				;;
			nt_chanceflurries)
				imageout=snow
				fontchar="O"
				;;
			nt_chancerain)
				imageout=ntpcrain
				fontchar="G"
				;;
			nt_chancesleet|nt_chancesnow|nt_flurries|nt_sleet)
				imageout=ntpcsnow
				fontchar="O"
				;;
			nt_chancetstorms)
				imageout=ntctstorm
				fontchar="K"
				;;
			nt_clear)
				imageout=ntclear
				fontchar="A"
				;;
			nt_cloudy)
				imageout=cloudy
				fontchar="f"
				;;
			nt_mostlycloudy)
				imageout=ntmcloudy
				fontchar="D"
				;;
			nt_partlycloudy)
				imageout=ntpcloudy
				fontchar="C"
				;;
			nt_rain)
				imageout=rain
				fontchar="G"
				;;
			nt_sleet)
				imageout=sleet
				fontchar="w"
				;;
			nt_snow)
				imageout=snow
				fontchar="q"
				;;
			nt_tstorms)
				imageout=tstorm
				fontchar="K"
				;;
			*|unknown|nt_unknown)
				imageout=na
				fontchar="-"
				;;
			# *)
				# imageout=na
				# fontchar="-"
				# ;;
		esac
		;;
	esac
		
	case $1 in
	1)
		cp ""$iwxpath"/"$imageout".png" "$nowCond"
		echo $fontchar
		;;
	2)
		cp ""$iwxpath"/"$imageout".png" "$temppath/daily-$3.png"
		echo $fontchar
		;;
	esac
}

# -----------------------------------------------------------------------
# Get moon state image n font character
# -----------------------------------------------------------------------
get_moon_image ()
{
	mphase="${moonp1,,}"
	case $percentilluminated in
	0)
		moonImage=30
		moonfontchar="${moon_font[29]}"
		;;
	100)
		moonImage=15
		moonfontchar="${moon_font[28]}"
		;;
	*)
		case $mphase in
		waxing|first)
			case $percentilluminated in
			[1-7]) # 1 - 7
				moonImage=01
				moonfontchar="${moon_font[0]}"
				;;
			[8-9]|1[0-4])  # 8 - 14
				moonImage=02
				moonfontchar="${moon_font[1]}"
				;;
			1[5-9]|2[0-1])  # 15 - 21
				moonImage=03
				moonfontchar="${moon_font[2]}"
				;;
			2[2-8])  # 22 - 28
				moonImage=04
				moonfontchar="${moon_font[3]}"
				;;
			29|3[0-5])  # 29 - 35
				moonImage=05
				moonfontchar="${moon_font[4]}"
				;;
			3[6-9]|4[0-2])  # 36 - 42
				moonImage=06
				moonfontchar="${moon_font[5]}"
				;;
			4[3-9])  # 43 - 49
				moonImage=07
				moonfontchar="${moon_font[6]}"
				;;
			5[0-6])  # 50 - 56
				moonImage=08
				moonfontchar="${moon_font[7]}"
				;;
			5[7-9]|6[0-3])  # 57 - 63
				moonImage=09
				moonfontchar="${moon_font[8]}"
				;;
			6[4-9]|70)  # 64 - 70
				moonImage=10
				moonfontchar="${moon_font[9]}"
				;;
			7[1-7])  # 71 - 77
				moonImage=11
				moonfontchar="${moon_font[10]}"
				;;
			7[8-9]|8[0-4])  # 78 - 84
				moonImage=12
				moonfontchar="${moon_font[11]}"
				;;
			8[5-9]|9[0-1])  # 85 - 91
				moonImage=13
				moonfontchar="${moon_font[12]}"
				;;
			9[2-9])  # 92 - 99
				moonImage=14
				moonfontchar="${moon_font[13]}"
				;;
			esac
			;;
		waning|last|third)
			case $percentilluminated in
			9[2-9])  # 92 - 99
				moonImage=16
				moonfontchar="${moon_font[14]}"
				;;
			8[5-9]|9[0-1])  # 85 - 91
				moonImage=17
				moonfontchar="${moon_font[15]}"
				;;
			7[8-9]|8[0-4])  # 78 - 84
				moonImage=18
				moonfontchar="${moon_font[16]}"
				;;
			7[1-7])  # 71 - 77
				moonImage=19
				moonfontchar="${moon_font[17]}"
				;;
			6[4-9]|70)  # 64 - 70
				moonImage=20
				moonfontchar="${moon_font[18]}"
				;;
			5[7-9]|6[0-3])  # 57 - 63
				moonImage=21
				moonfontchar="${moon_font[19]}"
				;;
			5[0-6])  # 50 - 56
				moonImage=22
				moonfontchar="${moon_font[20]}"
				;;
			4[3-9])  # 43 - 49
				moonImage=23
				moonfontchar="${moon_font[21]}"
				;;
			3[6-9]|4[0-2])  # 36 - 42
				moonImage=24
				moonfontchar="${moon_font[22]}"
				;;
			29|3[0-5])  # 29 - 35
				moonImage=25
				moonfontchar="${moon_font[23]}"
				;;
			2[2-8])  # 22 - 28
				moonImage=26
				moonfontchar="${moon_font[24]}"
				;;
			1[5-9]|2[0-1])  # 15 - 21
				moonImage=27
				moonfontchar="${moon_font[25]}"
				;;
			[8-9]|1[0-4])  # 8 - 14
				moonImage=28
				moonfontchar="${moon_font[26]}"
				;;
			[1-7]) # 1 - 7
				moonImage=29
				moonfontchar="${moon_font[27]}"
				;;
			esac
			;;
		full)
			moonImage=15
			moonfontchar="${moon_font[28]}"
			;;
		new)
			moonImage=30
			moonfontchar="${moon_font[29]}"
			;;
		*)
			moonImage=na
			moonfontchar="-"
			;;
		esac
		;;
	esac
	
	cp ""$imoonpath"/"$moonImage".png" "$nowMoon"
	echo "$moonfontchar"
	
}

# -----------------------------------------------------------------------
# Process the final template before showtime
# -----------------------------------------------------------------------
pre_showtime_template ()
{
	# set -x
	case $use_template_cleanup in
	1)
		local plen=${#template_cleanup[@]}
		for ((p=0; p<$plen; p++));
			do
				local bl_text=${template_cleanup[p]}
				if [[ $bl_text == *:* ]]; then
					local lt1=${bl_text/\:*}		# get value left of :
					local rt1=${bl_text#*\:}		# get value right of :
					local test1='s`'"$lt1"'`'"$rt1"'`g'
					local test2+="$test1;"
					err_1=0
				else
					err_1=1
				fi
			done
			
		case $err_1 in
		0)
			sed -i -e "${test2[@]}" $temp_template
			;;
		1)
			infomsg_stdout "$showtime_err"
			;;
		esac
		;;
	esac
	# set +x
}


# -----------------------------------------------------------------------
# Process the Current or Present Condition basis the blacklist
# in the config file
# -----------------------------------------------------------------------
present_cond_bl ()
{
	case $use_pcondition_blacklist in
	1)
		local plen=${#pcondition_blacklist[@]}
		local test_cond=${presentcondition,,}
		for ((p=0; p<$plen; p++));
			do
				local bl_text=${pcondition_blacklist[p],,}
				if [[ $bl_text == *:* ]]; then
					local lt1=${bl_text/\:*}		# get value left of :
					local rt1=${bl_text#*\:}		# get value right of :
					pcondition="${test_cond//$lt1/$rt1}"
				else
					pcondition="${test_cond//$bl_text/}"
				fi
				local test_cond="$pcondition"
			done
		;;
	*)
		pcondition="$presentcondition"
		;;
	esac
	
	echo $(do_pc_what_case "$pcondition")
}

do_pc_what_case ()
{
	# pc1=($(trim "$pcondition"))
	local pc1=($(trim "$1"))
	local pc_len=${#pc1[@]}
	case $pc_what_case in
		1) # to upper case
			total_str=${pc1[@]^^} # ; restchr=${restchr^^}
			;;
		2) # to lower case
			total_str=${pc1[@],,} # ; restchr=${restchr^^}
			;;
		3) # sentence case - First carachter in string to upper
			firstchr=${pc1:0:1}; firstchr=${firstchr^^}
			restchr=${pc1:1}; restchr=${restchr,,}
			restchr1=${pc1[@]:1}; restchr1=${restchr1,,}
			total_str="$firstchr$restchr $restchr1"
			;;
		*) # first char upper case each word block
			for ((q=0; q<$pc_len; q++));
			do
				pc2=${pc1[q]}
				firstchr=${pc2:0:1}; firstchr=${firstchr^^}
				restchr=${pc2:1}; restchr=${restchr,,}
				total_str+="$firstchr$restchr "
			done
		;;
	esac
			
	echo "$total_str"
	
}


# -----------------------------------------------------------------------
# Final output data process
# -----------------------------------------------------------------------
do_cl_opts ()
{
	local clopt=$1
	clopt="${clopt^^}" 
	case $clopt in
		PIF) # wx font
			# option "1" means is not daily image
			output=$( get_dailyimage "1" "$presentcondition" )
			;;
		PIC) # wx image
			output="$nowCond"
			;;
		PT) # Present Temp
			output="$( convert_units "$presenttemp" "temp" )"
			;;
		FL) # Feels Like Temp
			output=$( get_apparent_temp )
			;;
		TD) # today date
			output=$( date +"$today_date_format" )
			;;
		CT) # Present Condition
			output=$( present_cond_bl )
			;;
		RF) # (Rain Fall),
			output="$( convert_units "$rainfall" "distance" "4" )"
			;;
		SD) # (Snow Depth),
			case $Units_show in 
			1)
				output="$snowdepth in"
				;;
			0)
				output="$snowdepth"
				;;
			esac
			;;
		HM) # (Humidity),
			output="$humidity%"
			;;
		VI) # (Visibility),
			output="$( convert_units "$visibility" "distance" "3" )"
			;;
		WD) # (Wind Direction N, NW, WNW etc...), 
			output="$winddir"
			;;
		WS) # (Wind Speed),
			case $windspeed in
			Calm|N/A)
				output=$windspeed
				;;
			*)
				output="$( convert_units "$windspeed" "speed" "1" )"
				;;
			esac
			;;
		WG) # (Wind Gusts),
			output="$( convert_units "$windgust" "speed" "1" )"
			;;
		BF) # (Bearing Font), 
			output=$( get_bearing_font )
			;;
		SN) # (Observatory), 
			if [[ $net_connect == "on" ]]; then
				output=$stationname
			elif [[ $net_connect == "off" ]]; then
				output="NO CONNECTION!!! $stationname"
			fi
			;;
		BR) # (Barometer Reading)
			output="$( convert_units "$pressure" "pressure" )"
			;;
		UV) # (UV Index), 
			output="$uv2"
			;;
		UVT) # (UV Text), 
			output="$uvtext"
			;;
		DP) # (Dew Point), 
			output="$( convert_units "$dewpoint" "temp" )"
			;;
		SR) # (SunRise), 
			output=$srise
			;;
		SS) # (SunSet), 
			output=$sset
			;;
		LOV) # (DayLight),
			output="$lvlight"
			;;
		LOD) # (DayLight),
			output="$lofday"
			;;
		MP) # (Moon Phase), 
			output="$moonphase"
			;;
		MP1)
			output=$moonp1
			;;
		MP2)
			output=$moonp2
			;;
		MP3)
			case $moonp1 in
			New)
				output="New Moon"
				;;
			Full)
				output="Full Moon"
				;;
			First)
				output="First Qtr"
				;;
			Last)
				output="Last Qtr"
				;;
			*)
				output="$moonp1"
				;;
			esac
			;;
		MIF) # (Moon font character), 
			output=$( get_moon_image )
			;;
		MIC) # (Moon Image), 
			output="$nowMoon"
			;;
		MR) # (MoonRise),
			output=$mrise
			;;
		MS) # (MoonSet), 
			output=$mset
			;;
		NM) # newmoon
			output="$newmoon"
			;;
		FM) # fullmoon
			output="$fullmoon"
			;;
		MV) # Moon Visibility
			output="$percentilluminated%"
			;;
		LU) # (Last Update), 
			output="$lastupdate"
			;;
		LF) # (Last Fetch)
			output="$fetchtime"
			;;
		QOF) # (Quality of Forecast)
			output="$(do_pc_what_case $quality_of_fcast)"
			;;
		# Day 1 - shows today but by late evening switches to the next day
		D1D) # day 1 = today information
			output=$( get_short_date "${dayinfo[3]}" )
			;;
		D1T) # day 1 temperature
			output="$( convert_FC_units "${dayinfo[4]}" )"
			;;
		D1C) # day 1 rain prob
			output=${dayinfo[5]}
			;;
		D1F) # daily font
			# $1 - "2" means daily image, $2 is condition text, $3 is day num.
			output=$( get_dailyimage "2" "${dayinfo[5]}" 1 )
			;;
		D1I) # daily Image
			output="$temppath/daily-1.png" 
			;;
		D1P) # day 1 rain prob
			output=${dayinfo[6]}
			;;
		D2D) # day 1 = today information
			output=$( get_short_date "${dayinfo[7]}" )
			;;
		D2T) # day 1 temperature
			output="$( convert_FC_units "${dayinfo[8]}" )"
			;;
		D2C) # day 1 rain prob
			output=${dayinfo[9]}
			;;
		D2F)
			# $1 - "2" means daily image, $2 is condition text, $3 is day num.
			output=$( get_dailyimage "2" "${dayinfo[9]}" 2 ) 
			;;
		D2I) # daily Image
			output="$temppath/daily-2.png" 
			;;
		D2P) # day 1 rain prob
			output=${dayinfo[10]}
			;;
		D3D) # day 1 = today information
			output=$( get_short_date "${dayinfo[11]}" )
			;;
		D3T) # day 1 temperature
			output="$( convert_FC_units "${dayinfo[12]}" )"
			;;
		D3C) # day 1 rain prob
			output=${dayinfo[13]}
			;;
		D3F)
			# $1 - "2" means daily image, $2 is condition text, $3 is day num.
			output=$( get_dailyimage "2" "${dayinfo[13]}" 3 ) 
			;;
		D3I) # daily Image
			output="$temppath/daily-3.png" 
			;;
		D3P) # day 1 rain prob
			output=${dayinfo[14]}
			;;
		D4D) # day 1 = today information
			output=$( get_short_date "${dayinfo[15]}" )
			;;
		D4T) # day 1 temperature
			output="$( convert_FC_units "${dayinfo[16]}" )"
			;;
		D4C) # day 1 rain prob
			output=${dayinfo[17]}
			;;
		D4F)
			# $1 - "2" means daily image, $2 is condition text, $3 is day num.
			output=$( get_dailyimage "2" "${dayinfo[17]}" 4 ) 
			;;
		D4I) # daily Image
			output="$temppath/daily-4.png" 
			;;
		D4P) # day 1 rain prob
			output=${dayinfo[18]}
			;;
		D5D) # day 1 = today information
			output=$( get_short_date "${dayinfo[19]}" )
			;;
		D5T) # day 1 temperature
			output="$( convert_FC_units "${dayinfo[20]}" )"
			;;
		D5C) # day 1 rain prob
			output=${dayinfo[21]}
			;;
		D5F)
			# $1 - "2" means daily image, $2 is condition text, $3 is day num.
			output=$( get_dailyimage "2" "${dayinfo[21]}" 5 ) 
			;;
		D5I) # daily Image
			output="$temppath/daily-5.png" 
			;;
		D5P) # day 1 rain prob
			output=${dayinfo[22]}
			;;
		D6D) # day 1 = today information
			output=$( get_short_date "${dayinfo[23]}" )
			;;
		D6T) # day 1 temperature
			output="$( convert_FC_units "${dayinfo[24]}" )"
			;;
		D6C) # day 1 rain prob
			output=${dayinfo[25]}
			;;
		D6F)
			# $1 - "2" means daily image, $2 is condition text, $3 is day num.
			output=$( get_dailyimage "2" "${dayinfo[25]}" 6 ) 
			;;
		D6I) # daily Image
			output="$temppath/daily-6.png" 
			;;
		D6P) # day 1 rain prob
			output=${dayinfo[26]}
			;;
		D7D) # day 1 = today information
			output=$( get_short_date "${dayinfo[27]}" )
			;;
		D7T) # day 1 temperature
			output="$( convert_FC_units "${dayinfo[28]}" )"
			;;
		D7C) # day 1 rain prob
			output=${dayinfo[29]}
			;;
		D7F)
			# $1 - "2" means daily image, $2 is condition text, $3 is day num.
			output=$( get_dailyimage "2" "${dayinfo[29]}" 7 ) 
			;;
		D7I) # daily Image
			output="$temppath/daily-7.png" 
			;;
		D7P) # day 1 rain prob
			output=${dayinfo[30]}
			;;
		D8D) # day 1 = today information
			output=$( get_short_date "${dayinfo[31]}" )
			;;
		D8T) # day 1 temperature
			output="$( convert_FC_units "${dayinfo[32]}" )"
			;;
		D8C) # day 1 rain prob
			output=${dayinfo[33]}
			;;
		D8F)
			# $1 - "2" means daily image, $2 is condition text, $3 is day num.
			output=$( get_dailyimage "2" "${dayinfo[33]}" 8 )
			;;
		D8I) # daily Image
			output="$temppath/daily-8.png" 
			;;
		D8P) # day 1 rain prob
			output=${dayinfo[34]}
			;;
		FCTP) # pressure trend
			output=${ret_trends[0]}
			;;
		FCTT) # temperature trend
			output=${ret_trends[1]}
			;;
		FCTW) # wind trend
			output=${ret_trends[2]}
			;;
		FCTH) # humidity trend
			output=${ret_trends[3]}
			;;
		FCTU) # UV trend
			output=${ret_trends[4]}
			;;
		FCTD) # dewpoint trend
			output=${ret_trends[5]}
			;;
		FCTV) # visibility trend
			output=${ret_trends[6]}
			;;
		FCTSR) # sunrise trend
			output=${ret_trends[7]}
			;;
		FCTSS) # sunset trend
			output=${ret_trends[8]}
			;;
		FCTMR) # moonrise trend
			output=${ret_trends[9]}
			;;
		FCTMS) # moonset trend
			output=${ret_trends[10]}
			;;
		FCTLL) # length of visible light trend
			output=${ret_trends[11]}
			;;
		FCTLD) # length of day trend
			output=${ret_trends[12]}
			;;
		FCTMI) # percent moon illuminated trend
			output=${ret_trends[13]}
			;;
		FCTPF) # pressure trend font
			output=${ret_trends[14]}
			;;
		FCTTF) # temperature trend font
			output=${ret_trends[15]}
			;;
		FCTWF) # windspeed trend font
			output=${ret_trends[16]}
			;;
		FCTHF) # Humidity trend font
			output=${ret_trends[17]}
			;;
		FCTUF) # UV trend font
			output=${ret_trends[18]}
			;;
		FCTDF) # dewpoint trend font
			output=${ret_trends[19]}
			;;
		FCTVF) # visibility trend font
			output=${ret_trends[20]}
			;;
		FCTSRF) # sunrise trend font
			output=${ret_trends[21]}
			;;
		FCTSSF) # sunset trend font
			output=${ret_trends[22]}
			;;
		FCTMRF) # moonrise trend font
			output=${ret_trends[23]}
			;;
		FCTMSF) # moonset trend font
			output=${ret_trends[24]}
			;;
		FCTLLF) # length of visible light trend font
			output=${ret_trends[25]}
			;;
		FCTLDF) # length of day trend font
			output=${ret_trends[26]}
			;;
		FCTMIF) # percent moon illuminated  trend font
			output=${ret_trends[27]}
			;;
		METAR)
			output=$metar1
			;;
		PTBI)
			output=$path_to_background_image
			;;
		esac
		
		case $clopt in
		# check for negative values - for these items u cannot have a -ve value
		# so even if the website is crapped we do not need to see that
		RF|HM|VI|WS|WG|BR|UV)
			if [[ $( test_positive "$output" ) == "plus" ]]; then
				echo "$output"
			else
				echo "N/A"
			fi
			;;
		*)	
			case $output in
			""|[nN]ot*|[nN]/[aA])
				echo "N/A"
				;;
			*)
				echo "$output"
				;;
			esac
			;;
		esac
			
}


# -----------------------------------------------------------------------
# Finaly get the data out of production to sales
# -----------------------------------------------------------------------
process_data ()
{
	do_proc ()
	{
		echo -e "$1 - $( do_cl_opts $1 )"
	}
	
	case "$clopt1" in
	-b)
		# using file discriptors to only open output file once
		# otherwise file is opened and closed for each element
		exec 3<&1
		exec 1>>$wund_debug_log
		;;
	esac
	
	data_array=( PIF PIC PT FL TD CT RF SD HM VI WD WS WG BF SN BR UV UVT DP SR SS LOV LOD MP MP1 MP2 MP3 MIF MIC MR MS LU LF FM MV NM QOF D1D D1T D1C D1F D1I D1P D2D D2T D2C D2F D2I D2P D3D D3T D3C D3F D3I D3P D4D D4T D4C D4F D4I D4P D5D D5T D5C D5F D5I D5P D6D D6T D6C D6F D6I D6P D7D D7T D7C D7F D7I D7P D8D D8T D8C D8F D8I D8P FCTP FCTT FCTW FCTH FCTU FCTD FCTV FCTSR FCTSS FCTMR FCTMS FCTLL FCTLD FCTMI FCTPF FCTTF FCTWF FCTHF FCTUF FCTDF FCTVF FCTSRF FCTSSF FCTMRF FCTMSF FCTLLF FCTLDF FCTMIF METAR PTBI )
	
	dcount=${#data_array[@]}
	# infomsg_stdout $dcount
	case "$clopt1" in
	-t)
		if [[ -s $input_template ]]; then
			cp $input_template $temp_template
		else
			infomsg_stdout "$missing_template_msg"
			exit 0
		fi
		
		val1=()
		for ((c=0; c<$dcount; c++));
		do
			# val1=( "$( do_cl_opts ${data_array[$c]} )" )
			val1="$( do_cl_opts ${data_array[$c]} )"
			lookupterm="${data_array[$c]}"
			# using "`" character as delimiter
			sed -i -e 's`\['"$lookupterm"'\]`'"$val1"'`g' $temp_template
		done
		pre_showtime_template
		cat "$temp_template"
		;;
	-d)
		input_template="${input_template^^}"
		if [[ $( echo ${data_array[*]} | grep $input_template ) == "" ]]; then
			# infomsg_stdout "Invalid Element - Please check conkywx wiki for valid options."
			echo "[Err]"
		else
			echo "$( do_cl_opts $input_template )"
		fi
		;;
	-b)
		echo -e "$debug_msg" > $wund_debug_log
		
		echo "************* conkywx variables used *************" >> $wund_debug_log
		echo "
		config_file=$config_file
		use_xml=$use_xml 
		xml_ID=$xml_ID 
		Location=$Location 
		Units_to_Use=$Units_to_Use 
		Units_downloaded=$Units_downloaded
		Units_show=$Units_show 
		use_short_date=$use_short_date 
		show_short_cond_text=$show_short_cond_text
		show_notification=$show_notification 
		Time_format=$Time_format 
		today_date_format=$today_date_format 
		minimum_fetch_time=$minimum_fetch_time 
		main_url=$main_url 
		xml_url=$xml_url 
		fupdate=$fupdate 
		" >> $wund_debug_log
	
		echo "************* Today forecast - source Location data *************" >> $wund_debug_log
		cat $today_forecast >> $wund_debug_log

		echo "************* Astro forecast - source Location data *************" >> $wund_debug_log
		cat $astro_forecast1 >> $wund_debug_log
		OIFS="$IFS"; IFS=$'\n'; set -f
		dayinfo=($(sed -n '/viewCal/,/BestForecast/{s/<[^>]*>//g;:a;/</N;//ba;s/^[ \t]*//;s/[ \t]*$//;s/ \&.*//g;/Precipitation/d;/View Calendar/d;/BestForecast/d;/^$/d;p}' $currdata ))

		echo "************* Day Info - source Location data *************" >> $wund_debug_log
		echo "${dayinfo[*]}" >> $wund_debug_log

		if [[ $use_xml = "1" ]]; then
			today_xml_forecast=($(sed 's/^[ \t]*//;s/[ \t]*$//;/^$/d;/<\?xml/,/\/full/d;/<city/,/\/location/d;/weather/d; /heat_index/,/solar_radiation/d;/history_url/,/current_observation/d;s/<\/.*>//g;s/<//g; s/>/\n /g;s/^$/-/g' $currdata_2))
			echo "************* XML Info - source xml data *************" >> $wund_debug_log
			echo "${today_xml_forecast[*]}" >> $wund_debug_log
		fi
		set +f; IFS="$OIFS"

		echo "************* Element data output *************" >> $wund_debug_log
			for ((c=0; c<$dcount; c++));
			do
				do_proc ${data_array[$c]}
			done
			exec 1<&3
		echo
		echo "**************************************************************"
		echo "  DEBUG Information"
		echo "**************************************************************"
		echo "  DEBUG info for config file:"
		echo "  $config_file"
		echo "  "
		echo "  Please check the log file "
		echo "  $wund_debug_log"
		echo "  and attach the file with your message if required."
		echo " "
		echo "**************************************************************"
		echo
		;;
	esac
}


# -----------------------------------------------------------------------
# Process engineering
# -----------------------------------------------------------------------
do_main ()
{
	set_variables
	check_for_data
	get_present_data
	check_status
	ret_trends=( $(get_forecast_trends) )
	rm -rf $temppath/*.png
	process_data
}

conkywx_assistant ()
{
	ask_config ()
	{
		clear
		echo "
		To use Default config file - Just press Enter
		   The default config file will be picked up from
		   /etc/conkywx/conkywx.conf
		   or if exists from
		   /home/user-name/.config/conkywx/conkywx.conf
		
		OR
		
		Enter your config file path/name. Followed by Enter"
		read config_name
		# echo "$config_name"
		config_file=${config_name:-$Default_config_file}
	}
	
	confirm_delete ()
	{
		clear
		echo "
		Please confirm you want to:
		Reset $1 trend information ?
		
		Enter YES to confirm. Capital YES required !
		Press Enter on blank line to cancel."
		read confirm_del
		case $confirm_del in
		YES)
			case $1 in
			Hourly)
				rm -f $forecast_trends $forecast_trends_run
				;;
			Daily)
				rm -f $forecast_trends_time $forecast_trends_time24
				;;
			esac
			;;
		*)
			echo "Operation cancelled... exiting"
			# sleep 1
			;;
		esac
	}
	
	while [[ ! $qtype =~ ^[1,2,3,4,5,6,7,q]+$ ]];
	do
		clear
		echo "
		Conkywx Weather Program Assistant
		---------------------------------
		Current version of conkywx on your system is: $version
		Enter your option number: 
		  1 Quick HELP with conkywx weather program
		  2 Refer to offline conkywx WIKI Manual in your browser
		  3 FORCE update conkywx data
		  4 Get detailed DEBUG information for conkywx
		  5 Reset Hourly trend information
		  6 Reset Daily trend information
		  7 View Trend data table
		  
		  q to quit
		  followed by [ENTER]:
		  --------------------"
		  read qtype
	
		case $qtype in
		1)
			clear
			infomsg "conkywx version: $version"
			;;
		2)
			xdg-open "$cwxpath/conkywxwiki.html" >/dev/null 2>&1;
			;;
		3)
			fupdate=1
			ask_config
			do_main
			exit 0
			;;
		4)
			clopt1="-b"
			debug=1
			ask_config
			do_main
			exit 0
			;;
		5)
			ask_config
			set_variables
			confirm_delete "Hourly"
			exit 0
			;;
		6)
			ask_config
			set_variables
			confirm_delete "Daily"
			exit 0
			;;
		7)
			ask_config
			set_variables
			clear
			cat $viewtrends
			exit 0
			;;
		q)
			echo "Exiting conkywx assistant... have a fantastic day!!! ;-)"
			# sleep 1
			exit 0
			;;
		esac
	
	done
}


# -----------------------------------------------------------------------
# Lets do the do - command line options
# -----------------------------------------------------------------------
# do not remove this nowtime lines below !!!
# nowtime_testing=$( date +%s%N )

nowtime1=$( date +%s )

if [[ -s $HOME/.config/conkywx/conkywx.conf ]]; then
	Default_config_file="$HOME/.config/conkywx/conkywx.conf"
elif [[ -s $cwxpath/conkywx.conf ]]; then
	Default_config_file="$cwxpath/conkywx.conf"
elif [[ -s /etc/conkywx/conkywx.conf ]]; then
	Default_config_file="/etc/conkywx/conkywx.conf"
fi

# special declaration
debug=; fupdate=; clopt1=
# test for correct number of arguments and get values
if [ $# -eq 0 ]; then
	# help information
	echo ""
	conkywx_assistant
	#usage1
	exit 0
elif [ $# -gt 4 ]
	then
	infomsg "--- TOO MANY ARGUMENTS WERE PROVIDED ---"
else
	while [ $# -gt 0 ]
	do
		case "$1" in
		-h|--help)
			shift
			conkywx_assistant
			;;
		-v)
			shift
			conkywx_assistant
			;;
		--version)
			shift
			echo "$version"
			exit 0
			;;
		-c|--config)
			shift
			config_file_new="$1"
			;;
		-t|--template)
			shift
			input_template="$1"
			clopt1="-t"
			;;
		-d|--data)
			shift
			input_template="$1"
			clopt1="-d"
			;;
		*)
			infomsg "* * * Invalid options - try again. * * *"
			;;
		esac
		shift
	done
	
config_file=${config_file_new:-$Default_config_file}

do_main

fi

# script_time >> $time_test

exit

